import java.applet.Applet;
import java.awt.*;
import java.applet.*;
import java.math.*;
import java.awt.event.*;
import java.awt.geom.*;

public class DebugMaps {




    /**this tests the square recovery routine.*/
    public static void squareRecoveryTest(int p,int q) {
	System.out.println("start");
	for(int i=0;i<(p+q)*(p+q);++i) {
	    for(int j=0;j<p+q;++j) {
		boolean test=recoverTest(p,q,i,j);
	    }
	}
	System.out.println("done");
    }


    public static boolean recoverTest(int p,int q,double x,double y) {
	Complex z1=AGLattice.nearestGrid(p,q,x,y);
	Complex z2=squareRecovery(p,q,z1.x,z1.y);
	z1.x=z1.x-Math.floor(z1.x);
	z1.y=z1.y-Math.floor(z1.y);
	if(z1.x<.0000001) return true;
	if(z1.x>.9999999) return true;
	if(Complex.dist(z1,z2)>.0000001) {
	    System.out.println("bad");
	    z1.print();
	    z2.print();
            return false;
	}
	return true;
    }


    public static Complex squareRecovery(int p,int q,double x,double y) {
	double A=1.0*p/q;
	Complex z=new Complex(x,y);
	Complex w=AGTransforms.directMap(p,q,z);
	double[] d=AGMap.phiPlus(p,q,w.x,w.y);
	double x1=d[1];
	double y1=(d[0]-A*d[1]-1)/(1+A);
	if(y1<0) y1=y1+1;
	return(new Complex(x1,y1));
    }















    /**This tests various features of the classifying maps.
       The most important test is the square recovery test.*/

    public static void horizontalTest(int p,int q) {
	double A=1.0*p/q;
	int w=p+q;
	int xi=MathRational.tune2(p,q);
	//double[] d=AGMap.reduceLattice(A,2*xi*w,2*xi*w+1,0);
	double[] d=AGMap.reduceLattice(A,4*xi*p,1,0);
	d[0]=d[0]-1-A;
	System.out.println(d[0]+" "+d[1]+" "+d[2]);
    }





    /**This tests the map deltaPlus, which is used in the
       horizontal part of the hexagrid theorem*/

    public static void horizontalTest1(int p,int q) {
	int w=p+q;
	double max=-1;
        double A=1.0*p/q;

	for(int i=0;i<p+q;++i) {
	    for(int j=0;j<w*100;++j) {
		double jj=1.0*j/100;
	           double[] d=deltaPlus(p,q,jj,i);
	         
	           d=AGMap.reduceLattice(A,d[0],d[1],d[2]);
		   double key=d[1]-A*d[2]-1;
		   double test=Math.abs(key);
		   if(max<test) max=test;
	    }
	}
	System.out.println("max "+max);
    }





    /**currently computes the vertical partial derivative*/
    public static void horizontalTest2(int p,int q) {
	int w=p+q;
	double A=1.0*p/q;
	int h=(int)(w*Math.random());
	double dh=.0001;
	for(int i=0;i<1/dh;++i) {
	    double x1=1.0*i*dh;
	    double[] d1=deltaPlus(p,q,x1,h);
	    double[] d2=deltaPlus(p,q,x1,h+dh);
	    double[] d=new double[3];
	    for(int g=0;g<3;++g) {
		d[g]=(d2[g]-d1[g])/dh;
	    }
	    System.out.println(d[0]+" "+d[1]+" "+d[2]);
	    double test=1.0*(w*w+p*p)/(q*w);
	    System.out.println(test);
	    test=1.0*p/w;
	    System.out.println(test);
	}



    }



    public static double[] deltaPlus(int p,int q,double x,double y) {
	double A=1.0*p/q;
	Complex z=new Complex(x,y);
	Complex w=AGTransforms.directMap(p,q,z);
	double m=w.x;
	double n=w.y;
	double d1=2*A*(1-m+n)-m;
	double d2=A-d1;
	double d3=A*m;
	double[] d={d1,d2,d3};
	return(d);
    }


    /**This tests the images of the vertical lines*/
    public static void verticalTest(int p,int q) {
	double n=0*(p+q)+.0000000001;
	int level=(int)(1+p*Math.random());
	n=n+MathRational.tune(p,q,p+q-level);
	System.out.println("test "+n+"    "+p+"/"+q);
	double A=1.0*p/q;
	for(int i=0;i<100;++i) {
	    Complex z=new Complex(n,1.0*(i+0)/10);
	    Complex w=new Complex(n,1.0*(i+1)/10);
	    Complex z2=AGTransforms.directMap(p,q,z);
	    double[] d=AGMap.phiMinus(p,q,z2.x,z2.y);
	    double test1=d[1];
	    double test2=1+A-d[1];
	    test1=q*test1;
	    test2=q*test2;
	    System.out.println(level+"   "+test1+" "+test2);
	}

    }



    public static void cleanPrint(double[] d) {
	for(int i=0;i<d.length;++i) cleanPrint(d[i]);
	System.out.println("");
    }

    public static void cleanPrint(double d) {
	double d0=d;
	if(Math.abs(d)<.00000001) d0=0;
	System.out.print(d0+" ");
    }



}


