import java.awt.*;
import java.math.*;


public class Torus {

    /**These are the routines for the plaid model and
       for the cubical model.*/

    /**These take the decimal parts*/

    /**range is [0,1]*/

    public static double dec1(double x) {
	return(x-Math.floor(x));
    }

    /**range is [-1,1]*/

    public static double dec2(double x) {
	double y1=x/2.0+1/2.0;
	double y2=dec1(y1);
	double y3=2*y2-1;
	return(y3);
    }

    /**range is [-3,1]*/

    public static double dec4(double x) {
	double y1=x/4.0+1/2.0;
	double y2=dec1(y1);
	double y3=4*y2-2;
	return(y3);
    }


    /**Here P/Q=p/q and P+Q=2*/

    public static double P(int p,int q) {
	return(2.0*p/(p+q));
    }

    public static double Q(int p,int q) {
	return(2.0*q/(p+q));
    }

    /**Coordinates for the plaid model*/
    public static double FV(int p,int q,double x,double y) {
	double P=P(p,q);
	return dec2(2*P*x);
    }

    public static double FH(int p,int q,double x,double y) {
	double P=P(p,q);
	return dec2(2*P*y);
    }

    public static double FP(int p,int q,double x,double y) {
	double P=P(p,q);
	double t2=P*y+P*P*x+1;
	double t3=dec2(t2);
	return t3;
    }

    public static double FQ(int p,int q,double x,double y) {
	double P=P(p,q);
	double Q=Q(p,q);
	double t2=P*y+P*Q*x+1;
	double t3=dec2(t2);
	return t3;
    }


    public static double FPP(int p,int q,double x,double y) {
	double P=P(p,q);
	double t2=P*y-P*P*x+1;
	double t3=dec2(t2);
	return t3;
    }

    public static double FQP(int p,int q,double x,double y) {
	double P=P(p,q);
	double Q=Q(p,q);
	double t2=P*y-P*Q*x+1;
	double t3=dec2(t2);
	return t3;
    }



    /**Torus map coordinates*/


    public static double[] torusPoint(int p,int q,Complex Z) {
	return torusPoint(p,q,Z.x,Z.y);
    }

    public static double[] torusPoint2(int p,int q,Complex Z) {
	return torusPoint2(p,q,Z.x,Z.y);
    }



    public static double[] linearMap(int p, int q,double x, double y) {
	double P=P(p,q);
	double Q=Q(p,q);
	double a1=2*P*x+2*y;
	double a2=2*P*x;
	double a3=2*P*x+2*P*y;
	double[] X={P,a1,a2,a3};
	return(X);
    }

    public static int parity(int p,int q,double x,double y) {
	double[] X=torusPoint(p,q,x,y);
	if(X[1]<-1) return(1);
	if(X[1]>1) return(1);
	return(0);
    }


    public static double[] torusPoint(int p, int q,double x, double y) {
	double[] X=linearMap(p,q,x,y);
	X=reduce(p,q,X);
	return(X);
    }

    public static double[] torusPoint2(int p, int q,double x, double y) {
	double[] X=linearMap(p,q,x,y);
	X=reduce2(p,q,X);
	return(X);
    }



    public static double[] reduce(int p,int q,double[] u) {
	double P=P(p,q);
	double x=u[1]/4;
	x=Math.floor(x+.5);
	x=4*x;
	double[] v={u[0],u[1],u[2],u[3]};
	v[1]=u[1]-x;
	v[2]=u[2]-x*P/2;
	v[3]=u[3]-x*P/2;
	v[2]=dec2(v[2]);
	v[3]=dec2(v[3]);
	return(v);
    }

    public static double[] reduce2(int p,int q,double[] u) {
	double P=P(p,q);
	double x=u[1]/2;
	x=Math.floor(x+.5);
	x=2*x;
	double[] v={u[0],u[1],u[2],u[3]};
	v[1]=u[1]-x;
	v[2]=u[2]-x*P/2;
	v[3]=u[3]-x*P/2;
	v[2]=dec2(v[2]);
	v[3]=dec2(v[3]);
	return(v);
    }


    /**These are some of the local formulas described in the paper.*/


    public static double[] torusPoint2Alt(int p, int q,double x, double y) {
	double P=P(p,q);
	double Q=Q(p,q);
	double a1=dec2(1+2*P*x);
	double c=a1*P/2;
	double a2=dec2(c-P*y+P*Q*x);
	double a3=dec2(c+P*y+P*Q*x);
	double[] X={P,a1,a2,a3};
	return(X);
    }

    public static double[] torusPointAlt(int p, int q,double x, double y) {
	double P=P(p,q);
	double Q=Q(p,q);
	double a1=dec4(2*y+2*P*x);
	double c=a1*P/2;
	double a2=dec2(c-P*y+P*Q*x);
	double a3=dec2(c+P*y+P*Q*x);
	double[] X={P,a1,a2,a3};
	return(X);
    }

}

