import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;
import java.util.Arrays;


/**This class generates the polyhedra in the plaid
and graph partitions.  It also checks various
things about the partition.*/

public class DebugPartition implements Runnable {
    boolean HALT=true;
    int choice;
    int dir;



    public void run() {
	printGraph14();
    }








    public void printAssignment() {
	for(int i=0;i<218;++i) {
	    int[] code=DataTripleCodes.data(i);
	    int[][] typeP=DataTripleIndex.typePlus(i);
	    int[][] typeM=DataTripleIndex.typeMinus(i);
	    if((code[0]==2)&&(code[1]==1)) {
		System.out.println("------"+i+"-----");
	    }
	}

    }


    public void printout(int[][] t) {
	for(int i=0;i<t.length;++i) {
	    System.out.print(t[i][0]+" "+t[i][1]+"   ");
	}
	System.out.println("");
    }


    /**This prints out the 14 graph polytopes in the + partition*/

    public void printGraph14() {
	Output OUT=new Output("graph_polytopes.tex");
	for(int L=1;L<20;++L) {
	for(int i=0;i<14;++i) {	
            PolytopeLong P=AllPolytopesLong.graph(i);
	    P=downScale(P);
	    if(P.count==L) {print2(OUT,P);
	       OUT.writeln("SLASHvspace{2mm}");
	       OUT.writeln("");
	    }
	}}
    }


    /**This checks the relation between the forward and
       backwards partition, using the map graphF2B*/

    public void checkGraphF2B() {
	for(int i=0;i<14;++i) {
	    PolytopeLong P=AllPolytopesLong.graph(i);
	    PolytopeLong Q=AllPolytopesLong.graph(14+i);
	    P=downScale(P);
	    P=graphF2B(P);
	    Q=downScale(Q);
	    boolean test=match(P,Q);
	    System.out.println(test);
	}
    }


    public PolytopeLong graphF2B(PolytopeLong P) {
	PolytopeLong Q=new PolytopeLong();
	Q.count=P.count;
	for(int i=0;i<P.count;++i) {
	    long x0=P.V[i].x[0];
	    long x1=P.V[i].x[1];
	    long x2=P.V[i].x[2];
	    long x3=P.V[i].x[3];
   	    Q.V[i]=new Vector4Long(1-x0,-x1+x3,2-x2+x3,x3);
	}
	return Q;
    }




    /**This checks that the 10 fundamental polytopes generate
       the rest under the dihedral action of negating and flipping.*/

    public void plaidFundamental() {
	System.out.println("start");
	int[] list={7,18,19,20,21,22,23,24,26,27};
	for(int i=0;i<28;++i) {
	    PolytopeLong P=AllPolytopesLong.plaid(i);
  	    P=downScale(P);
	    boolean test=false;
	    for(int j=0;j<list.length;++j) {
		PolytopeLong Q=AllPolytopesLong.plaid(list[j]);
		Q=downScale(Q);
		PolytopeLong Q1=negate(Q);
		PolytopeLong Q2=flip(Q);
		PolytopeLong Q3=flip(Q1);
		if(match(P,Q)==true) test=true;
		if(match(P,Q1)==true) test=true;
		if(match(P,Q2)==true) test=true;
		if(match(P,Q3)==true) test=true;
	    }
	    if(test==false) throw new ProofException("blah");
	}
	System.out.println("done");
    }



    /**This prints out the 10 plaid polytopes*/

    public void printPlaid10() {
	Output OUT=new Output("plaid_polytopes.tex");
	int[] list={7,18,19,20,21,22,23,24,26,27};
	for(int L=1;L<20;++L) {
	for(int i=0;i<10;++i) {	
            PolytopeLong P=AllPolytopesLong.plaid(list[i]);
	    P=downScale(P);
	    if(P.count==L) {print(OUT,P);
	       OUT.writeln("SLASHvspace{2mm}");
	       OUT.writeln("");
	    }
	}}
    }




    public PolytopeLong downScale(PolytopeLong P) {
	PolytopeLong Q=new PolytopeLong();
	Q.count=P.count;
	for(int i=0;i<P.count;++i) {
   	    Q.V[i]=new Vector4Long();
	    for(int j=0;j<4;++j) {
		Q.V[i].x[j]=P.V[i].x[j]/60;
	    }
	}
	Q.type[1]=P.type[1];
	Q.type[2]=P.type[2];
	return Q;
    }

    public PolytopeLong negate(PolytopeLong P) {
	PolytopeLong Q=new PolytopeLong();
	Q.count=P.count;
	for(int i=0;i<P.count;++i) {
   	    Q.V[i]=new Vector4Long(-P.V[i].x[0],-P.V[i].x[1],-P.V[i].x[2],P.V[i].x[3]);
	}
	return Q;
    }



    public PolytopeLong flip(PolytopeLong P) {
	PolytopeLong Q=new PolytopeLong();
	Q.count=P.count;
	for(int i=0;i<P.count;++i) {
   	    Q.V[i]=new Vector4Long(P.V[i].x[0],P.V[i].x[2],P.V[i].x[1],P.V[i].x[3]);
	}
	return Q;
    }


    public boolean match(PolytopeLong P,PolytopeLong Q) {
	if(P.count!=Q.count) return false;
	for(int i=0;i<P.count;++i) {
	    boolean test=false;
	    for(int j=0;j<Q.count;++j) {
		Vector4Long R=Vector4Long.minus(P.V[i],Q.V[j]);
		if(R.isZero()==true) test=true;
	    }
	    if(test==false) return false;
	}
	return true;
    }



    /**The SLASH needs to be changed to a backslash in the file.
       For some reason I can't include this symbol in the routine.
       It messes things up.*/

    public void print(Output OUT,PolytopeLong P) {
        OUT.write("SLASHlefteqn{");   //start equation
	for(int i=0;i<P.count;++i) {
	    OUT.write("SLASHleft[SLASHmatrix{");
	    int a0=(int)(P.V[i].x[0]);
	    int a1=(int)(P.V[i].x[1]);
	    int a2=(int)(P.V[i].x[2]);
	    int a3=(int)(P.V[i].x[3]);

	    OUT.write(a0+" SLASHcr "+a1+" SLASHcr "+a2+" SLASHcr "+a3+"}");
  	    OUT.writeln("SLASHright]");
	    if(i<P.count-1) OUT.write("SLASHhskip 5 pt");
	}   

	String[] S={"S","W","N","E"};

            OUT.write("SLASHhskip 20 pt ");
            String S1=S[P.type[1]];
	    String S2=S[P.type[2]];
	    if((P.type[1]==0)&&(P.type[2]==0)) {
		S1="X";
		S2="";
	    }
	    OUT.write("("+S1+",");
	    OUT.write(S2+")");

	    OUT.writeln("}");    //end equation
	System.out.println("");
    }

    public void print2(Output OUT,PolytopeLong P) {
        OUT.write("SLASHlefteqn{");   //start equation
	for(int i=0;i<P.count;++i) {
	    OUT.write("SLASHleft[SLASHmatrix{");
	    int a0=(int)(P.V[i].x[0]);
	    int a1=(int)(P.V[i].x[1]);
	    int a2=(int)(P.V[i].x[2]);
	    int a3=(int)(P.V[i].x[3]);

	    OUT.write(a0+" SLASHcr "+a1+" SLASHcr "+a2+" SLASHcr "+a3+"}");
  	    OUT.writeln("SLASHright]");
	    if(i<P.count-1) OUT.write("SLASHhskip 5 pt");
	}   


            OUT.write("SLASHhskip 20 pt ");
	    Integer I1=new Integer(P.type[1]);
	    Integer I2=new Integer(P.type[2]);
	    String S1=I1.toString();
	    String S2=I2.toString();

	    if((P.type[1]==0)&&(P.type[2]==0)) {
		S1="X";
		S2="";
	    }
	    OUT.write("("+S1+",");
	    OUT.write(S2+")");

	    OUT.writeln("}");    //end equation
	System.out.println("");
    }

}

