import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;


/**This checks that various of the polyhedra are disjoint from
   various other ones.*/


public class ProofDisjoint {


    public static void main() {
	testGraphPlus();
	testGraphMinus();
	testDoublePlus();
	testDoubleMinus();
	testPlaid();
	testPlaidTriple();
	testVolumePlaid();
	testVolumeGraph();
    }


    /**This tests that all the pieces in the (+) graph partition have
       pairwise disjoint interiors.  We already tested this for our
       Master Picture Theorem, but there's no harm doing it again.*/

    public static void testGraphPlus() {
	System.out.print("graph plus test: start  ");
	for(int i=0;i<14;++i) {
	    for(int j=i+1;j<14;++j) {
		PolytopeLong P=AllPolytopesLong.graph(i);
		PolytopeLong Q0=AllPolytopesLong.graph(j);
		for(int k0=-3;k0<=3;++k0) {
		for(int k1=-3;k1<=3;++k1) {
		for(int k2=-3;k2<=3;++k2) {
		    PolytopeLong Q=AllPolytopesLong.graphShift(k0,k1,k2,Q0);
		    boolean test=P.isDisjoint(Q);
		    if(test==false) throw new ProofException("graph plus disjointness fails");
		}}}
	    }
	}
	System.out.println("done");
    }

    /**This tests that all the pieces in the (-) graph partition have
       pairwise disjoint interiors.  We already tested this for our
       Master Picture Theorem, but there's no harm doing it again.*/


    public static void testGraphMinus() {
	System.out.print("graph minus test: start  ");
	for(int i=14;i<28;++i) {
	    for(int j=i+1;j<28;++j) {
		PolytopeLong P=AllPolytopesLong.graph(i);
		PolytopeLong Q0=AllPolytopesLong.graph(j);
	        for(int k0=-3;k0<=3;++k0) {
		for(int k1=-3;k1<=3;++k1) {
		for(int k2=-3;k2<=3;++k2) {
		    PolytopeLong Q=AllPolytopesLong.graphShift(k0,k1,k2,Q0);
		    boolean test=P.isDisjoint(Q);
		    if(test==false) throw new ProofException("graph plus disjointness fails");
		}}}
	    }
	}
	System.out.println("done");
    }



    public static void testPlaid() {
	System.out.print("plaid: start  ");
	for(int i=0;i<26;++i) {
	    for(int j=i+1;j<26;++j) {
		PolytopeLong P=AllPolytopesLong.plaid(i);
		PolytopeLong Q0=AllPolytopesLong.plaid(j);
	        for(int k0=-3;k0<=3;++k0) {
		for(int k1=-3;k1<=3;++k1) {
		for(int k2=-3;k2<=3;++k2) {
		    PolytopeLong Q=AllPolytopesLong.plaidShift(k0,k1,k2,Q0);
		    boolean test=P.isDisjoint(Q);
		    if(test==false) {  
			System.out.println(i+" "+j+"   "+k0+" "+k1+" "+k2);
                        throw new ProofException("plaid disjointness fails");
		    }
		}}}
	    }
	}
	System.out.println("done");
    }



    /**This tests that the each double from the (+) partition is disjoint from all the other partition pieces except for the two that comprise it.  At the same time, it tests that the two exceptional pieces are contained in the double.*/


    public static void testDoublePlus() {
	System.out.println("test disjointness doubles +");
	for(int i=0;i<4;++i) {
	    for(int j0=-3;j0<=3;++j0) {
		for(int j1=-3;j1<=3;++j1) {
	          for(int j2=-3;j2<=3;++j2) {
		      for(int j3=0;j3<14;++j3) {
		         PolytopeLong P=AllPolytopesLong.graphDouble(i);
		         PolytopeLong Q=AllPolytopesLong.graph(j3);
		         Q=AllPolytopesLong.graphShift(j0,j1,j2,Q);
		         boolean test=P.isDisjoint(Q);
		         if(test==false) {
			     boolean test2=P.contains(Q);
                             System.out.println(i+"    "+j0+" "+j1+" "+j2+" "+j3+"   "+test2);
			 }
		      }
		  }
		}
	    }	
	}
	System.out.println("done");
	System.out.println("");
    }

    /**This does the same thing for the (-) partition*/

    public static void testDoubleMinus() {
	System.out.println("test disjointness doubles -");
	for(int i=4;i<9;++i) {
	    for(int j0=-1;j0<3;++j0) {
		for(int j1=-2;j1<3;++j1) {
	          for(int j2=-1;j2<2;++j2) {
		      for(int j3=14;j3<28;++j3) {
		         PolytopeLong P=AllPolytopesLong.graphDouble(i);
		         PolytopeLong Q=AllPolytopesLong.graph(j3);
		         Q=AllPolytopesLong.graphShift(j0,j1,j2,Q);
		         boolean test=P.isDisjoint(Q);
		         if(test==false) { 
                              boolean test2=P.contains(Q);
                              System.out.println(i+"    "+j0+" "+j1+" "+j2+" "+j3+"   "+test2);
			 }
		      }
		  }
		}
	    }
	}
	System.out.println("done");
    }


    /**This tests that all the pieces in the plaid triple partition
       pairwise disjoint interiors.*/

    public static void testPlaidTriple() {
	System.out.print("plaid triple test: start  ");
	for(int i=0;i<218;++i) {
	    for(int j=i+1;j<218;++j) {
		PolytopeLong P=AllPolytopesLong.plaidTriple(i);
		PolytopeLong Q=AllPolytopesLong.plaidTriple(j);
		boolean test=P.isDisjoint(Q);
		if(test==false) throw new ProofException("plaid disjointness fails");
	    }
	}
	System.out.println("done");
    }


    /**This checks that the union of the pieces in the plaid
       partition has the same volume as the volume of the
       lattice fundamental domain. This is just an extra check.
       Technically, we need to add two translates of the plaid
       polygons to get a full fundamental domain.*/

    public static void testVolumePlaid() {
	long total=0;
	for(int i=0;i<26;++i) {
	   PolytopeLong P=AllPolytopesLong.plaid(i);
	   long vol=PolytopeLongVolume.volume(P);
	   long test=vol%480;
	   if(test!=0) {
              throw new ProofException("not divisible by 480");
	   }
	   vol=vol/480;
	   total=total+vol;
	}
	System.out.println("total   "+total);
	long target=12*120*120*60;
	System.out.println("target  "+target);
    }




    public static void testVolumeGraph() {
	long total=0;
	for(int i=0;i<14;++i) {
	   PolytopeLong P=AllPolytopesLong.graph(i);
	   long vol=PolytopeLongVolume.volume(P);
	   long test=vol%480;
	   if(test!=0) {
              throw new ProofException("not divisible by 480");
	   }
	   vol=vol/480;
	   total=total+vol;
	}
	System.out.println("total   "+total);
	long target=112*60*60*60*60/480;
	System.out.println("target  "+target);
    }


}


