import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;


/**Here are the volume calculations for the three partitions*/


public class ProofVolume {


    public static void main() {
	testVolumePlaid();
	testVolumeGraph();
	testVolumePlaidTriple();
    }



    /**This checks that the union of the pieces in the plaid
       partition has the same volume as the volume of the
       lattice fundamental domain. This is just an extra check.
       Technically, we need to add two translates of the plaid
       polygons to get a full fundamental domain.*/

    public static void testVolumePlaid() {
	long total=0;
	for(int i=0;i<26;++i) {
	   PolytopeLong P=AllPolytopesLong.plaid(i);
	   long vol=PolytopeLongVolume.volume(P);
	   long test=vol%480;
	   if(test!=0) {
              throw new ProofException("not divisible by 480");
	   }
	   vol=vol/480;
	   total=total+vol;
	}
	System.out.println("total   "+total);
	long target=12*120*120*60;
	System.out.println("target  "+target);
    }



    /**This checks that the union of the pieces in the graph
       partition has the same volume as the volume of the
       lattice fundamental domain  We just check the (+)
       partition.  The (-) partition is isometric*/


    public static void testVolumeGraph() {
	long total=0;
	for(int i=0;i<14;++i) {
	   PolytopeLong P=AllPolytopesLong.graph(i);
	   long vol=PolytopeLongVolume.volume(P);
	   long test=vol%480;
	   if(test!=0) {
              throw new ProofException("not divisible by 480");
	   }
	   vol=vol/480;
	   total=total+vol;
	}
	System.out.println("total   "+total);
	long target=112*60*60*60*60/480;
	System.out.println("target  "+target);
    }


    /**This checks that the union of the pieces in the triple
       partition has the same volume as the volume of the
       lattice fundamental domain.  To avoid overflow errors
       we divide all the volumes by 10.  Really, we are computing
       48 times the volume, and then dividing by 480.*/

    public static void testVolumePlaidTriple() {
	long total=0;
	for(int i=0;i<218;++i) {
	PolytopeLong P=AllPolytopesLong.plaidTriple(i);
	long vol=PolytopeLongVolume.volume(P);
	long test=vol%480;
	if(test!=0) {
            throw new ProofException("not divisible by 480");
	}
	vol=vol/480;
	total=total+vol;
	}
	System.out.println("total   "+total);
	long target=12*120*120*60;
	System.out.println("target  "+target);
    }





}


